
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Carroll, Otsuka and Slacalek: "How Large Are Financial and Housing Wealth Effects?: A New Approach"
% January 30, 2009
% 
% This program simulates the reaction of consumption and wealth to a shock to wealth 
% followed by a shock to income growth under frictionless and sticky expectations, and estimates WE regressions.
% The experiment is described in simulationsWithStickyC.pdf
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all; 
thispath = mfilename('fullpath'); thisfile = mfilename;
pathlength = size(thispath,2); namelength = size(thisfile,2);
CurrDir = thispath(1:(pathlength-namelength));
basePath=CurrDir(1:( length(CurrDir)-length('theoretical\matlab\') ));
cd(CurrDir);
diaryName=[mfilename '.out']; delete(diaryName);  diary(diaryName); 

disp('==================================================================================');
disp(['Running  ' mfilename '.m                      ' datestr(now)]); format compact; tic;
disp('==================================================================================');

% % My baseline setup
disp('Jirka''s baseline setup');
rho=2               % relative risk aversion
g=(1.015)^.25-1     % income growth (in the first regime)
beta=.99;           % discount factor
R=(1+g)^2/beta      % To make sure C=Y during the first phase
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PiFrac=0.25         % Fraction of updaters;
BLev0=0;
YLev0=1;
nPersStart=5; nPers=20
nPersPlusStart=nPersStart+nPers;
totalNpers=2*nPers+nPersStart;

r=R-1;
G=1+g;
G1=(1.025)^.25;     % income growth (in the first regime)
mpc=(R-(R*beta)^(1/rho))/R; 

BLevShock=1; % times initial income
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialize matrices;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
j=0;
perAfterShock=0;    % initialize counter for periods at and after 1st shock
fracNonupd=1;       % initialize counter for the fraction of perfect nonupdaters
OLev=repmat(nan,totalNpers,1); CLev=OLev; YLev=OLev; HLev=OLev; BLev=OLev;
BLev(1)=BLev0; YLev(1)=YLev0;
BbarLev=OLev; CbarLev=OLev; CratLev=OLev; YbarLev=OLev; HbarLev=OLev; ObarLev=OLev; BbarLev=OLev; 
BbarLev(1)=BLev0; YbarLev(1)=YLev0;

%====================================================================
% Shock to market wealth
%====================================================================
for i=1:nPersPlusStart; % Loop over time;

    if i==nPersStart; % 
        BLev(i)=BLev(i)+BLevShock;            
        BbarLev(i)=BbarLev(i)+BLevShock;
        perAfterShock=1;
    end;

    HLev(i)=YLev(i)/(1-G/R);
    OLev(i)=BLev(i)+HLev(i);
    CLev(i)=mpc*OLev(i);

    YLev(i+1)=G*YLev(i);
    BLev(i+1)=R*(BLev(i)+YLev(i)-CLev(i));
    
    HbarLev(i)=YbarLev(i)/(1-G/R);
    ObarLev(i)=BbarLev(i)+HbarLev(i);
    CratLev(i)=mpc*ObarLev(i);    % rational C for those living in the sticky world
    
    if i==1; CbarLev(i)=CLev(i); 
    else; CbarLev(i)=PiFrac*CratLev(i)+(1-PiFrac)*((R*beta)^(1/rho))*CbarLev(i-1);
    end;
    
    YbarLev(i+1)=G*YbarLev(i);
    BbarLev(i+1)=R*(BbarLev(i)+YbarLev(i)-CbarLev(i));
    fracNonupd=(1-PiFrac)*fracNonupd*perAfterShock+(1-perAfterShock)*fracNonupd;
    j=j+1;  %counter
end;
disp('Fraction of perfect non-updaters after the first shock:'); fracNonupd

% Normalize with income;
CRatRat=CratLev./YLev; CRat=CLev./YLev; BRat=BLev./YLev;
CBarRat=CbarLev./YLev; BBarRat=BbarLev./YLev; OBarRat=ObarLev./YLev;

%====================================================================
% Shock to income growth
%====================================================================
% split people between perfect nonupdaters and the rest ("non-nonupdaters"/nnupd)
OnnupdLev=repmat(nan,nPers,nPers+1); % Rows: time periods; columns: inattention cohorts (most attentive on the left);
% "nnupd" stands for nonupdaters
BnnupdLev=OnnupdLev; CnnupdLev=OnnupdLev; YnnupdLev=OnnupdLev; HnnupdLev=OnnupdLev; Bact_nnupdLev=OnnupdLev;
OnupdLev=OnnupdLev; BnupdLev=OnupdLev; CnupdLev=OnupdLev; YnupdLev=OnupdLev; HnupdLev=OnupdLev; Bact_nupdLev=OnupdLev;
YnormLev=OnnupdLev; YnormLev(1,:)=YLev(nPersPlusStart); % actual income (for normalization)

% Paste the series
% Nonupdaters
Cnupd_valLev=(((R*beta)^(1/rho))^(nPersPlusStart-1))*CLev(1);
Bnupd_valLev=-HLev(nPersPlusStart)+Cnupd_valLev/mpc; % invert consumption function to get perceived total resources for nonupdaters
BnupdLev(1,:)=Bnupd_valLev;

Bact_nupdLev(1,:)=BLevShock*R^nPers;
YnupdLev(1,:)=YbarLev(nPersPlusStart); 

% Updaters, ie non-nonupdaters [Note: don't need Cnnupd]
BnnupdLev(1,:)=BbarLev(nPersPlusStart); 
Bact_nnupdLev=BnnupdLev; % Tact stores actual market wealth of the inattentive (not perceived by them until they update)
YnnupdLev(1,:)=YbarLev(nPersPlusStart); 

G0=G;
for i=1:nPers; % Loop over time;
    for j=1:(nPers+1); % Loop over inattention spans
       
        if i==j;
           % Do nonnonupdaters first
           %==================================================================
           BnnupdLev(i,j)=Bact_nnupdLev(i,j);   
           YnnupdLev(i,j)=YnnupdLev(i,j)*((G1/G0)^(j-1));
           
           % Then do perfect nonupdaters
           %==================================================================
           BnupdLev(i,j)=Bact_nupdLev(i,j)+(BLevShock)*R^(nPers+j-1);   
           YnupdLev(i,j)=YnupdLev(i,j)*((G1/G0)^(j-1)); 
        end;
        
        if i>=j; G=G1; else; G=G0; end; % (iii) [and (ii)];
        
        HnnupdLev(i,j)=YnnupdLev(i,j)/(1-G/R);
        OnnupdLev(i,j)=BnnupdLev(i,j)+HnnupdLev(i,j); 
        CnnupdLev(i,j)=mpc*OnnupdLev(i,j);

        YnnupdLev(i+1,j)=G*YnnupdLev(i,j);
        BnnupdLev(i+1,j)=R*(BnnupdLev(i,j)+YnnupdLev(i,j)-CnnupdLev(i,j));
        Bact_nnupdLev(i+1,j)=R*(Bact_nnupdLev(i,j)+YnnupdLev(i,1)-CnnupdLev(i,j));    % Note that Y(i,1) is the actual income (of frictionless consumers);
        
        %====================================================================== 
        HnupdLev(i,j)=YnupdLev(i,j)/(1-G/R);
        OnupdLev(i,j)=BnupdLev(i,j)+HnupdLev(i,j); 
        CnupdLev(i,j)=mpc*OnupdLev(i,j);

        YnupdLev(i+1,j)=G*YnupdLev(i,j);
        BnupdLev(i+1,j)=R*(BnupdLev(i,j)+YnupdLev(i,j)-CnupdLev(i,j));
        Bact_nupdLev(i+1,j)=R*(Bact_nupdLev(i,j)+YnupdLev(i,1)-CnupdLev(i,j));    % Note that Y(i,1) is the actual income (of frictionless consumers);    
        
        YnormLev(i+1,j)=G1*YnormLev(i,j);
    end;
    YLev(i+nPersPlusStart)=G1*YLev(i+nPersPlusStart-1);
end;

%====================================================================
% Normalize with income;
BNnupdRat=BnnupdLev(:,1:end-1)./YnormLev(:,1:end-1); CNnupdRat=CnnupdLev(:,1:end-1)./YnormLev(1:end-1,1:end-1); 
HNnupdRat=HnnupdLev(:,1:end-1)./YnormLev(1:end-1,1:end-1); ONnupdRat=OnnupdLev(:,1:end-1)./YnormLev(1:end-1,1:end-1); 
YNnupdRat=YnnupdLev(:,1:end-1)./YnormLev(:,1:end-1);
BNupdRat=BnupdLev(:,1:end-1)./YnormLev(:,1:end-1); CNupdRat=CnupdLev(:,1:end-1)./YnormLev(1:end-1,1:end-1); 
HNupdRat=HnupdLev(:,1:end-1)./YnormLev(1:end-1,1:end-1); ONupdRat=OnupdLev(:,1:end-1)./YnormLev(1:end-1,1:end-1); 
YNupdRat=YnupdLev(:,1:end-1)./YnormLev(:,1:end-1);
BAct_nupdRat=Bact_nupdLev(:,1:end-1)./YnormLev(:,1:end-1); BAct_nnupdRat=Bact_nnupdLev(:,1:end-1)./YnormLev(:,1:end-1);
time=[1:totalNpers]';

% Sum over consumers to get the effect on aggregate vars;
auxVector=[1; (1-PiFrac)*ones(nPers-1,1)];
consWeights=PiFrac*cumprod(auxVector); consWeights=repmat(consWeights',nPers,1);
auxVectorC=[1; ((R*beta)^(1/rho))*ones(nPers-1,1)];

CAggrNnRat=sum((CNnupdRat.*consWeights)')'+(1-sum(consWeights(1,:)))*CnnupdLev(:,end)./YnormLev(1:end-1,end); % Adjust for the cutoff in weights
CAggrNupdRat=sum((CNupdRat.*consWeights)')'+(1-sum(consWeights(1,:)))*CnupdLev(:,end)./YnormLev(1:end-1,end);

BAdjNnRat=(1-sum(consWeights(1,:)))*Bact_nnupdLev(2:end,end)./YnormLev(2:end,1);
BAdjNupdRat=(1-sum(consWeights(1,:)))*Bact_nupdLev(2:end,end)./YnormLev(2:end,1);
BAggrNnRat=sum((BAct_nnupdRat(2:end,:).*consWeights)')'+BAdjNnRat; 
BAggrNupdRat=sum((BAct_nupdRat(2:end,:).*consWeights)')'+BAdjNupdRat; % This adjustment is zero anyway
CAggrRat=(1-fracNonupd)*CAggrNnRat+fracNonupd*CAggrNupdRat;
BAggrRat=(1-fracNonupd)*BAggrNnRat+fracNonupd*BAggrNupdRat;
BFlessRat=BNnupdRat(:,1);
BFlessRat=BFlessRat(2:end)*BRat(nPersPlusStart)/BFlessRat(1);  % re-normalize to get the level right
%CFlessRat=CNnupdRat(:,1);  % rational consumers, who are perfectly up-to-date
HFlessRat=HNupdRat(:,1);
CFlessRat=mpc*(BFlessRat+HFlessRat);  % rational consumers, who are perfectly up-to-date

% Paste the series for the two shocks together
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
CAllRat=[CBarRat(1:nPersPlusStart); CAggrRat];
BAllRat=[BBarRat(1:nPersPlusStart); BAggrRat];
CAll_flessRat=[CRat(1:nPersPlusStart); CFlessRat];
BAll_flessRat=[BRat(1:nPersPlusStart); BFlessRat];

CallLev=CAllRat.*YLev; BallLev=BAllRat.*YLev;
dCall=diff(CallLev); dBall=diff(BallLev);

% Run regressions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('OLS Regression of diffC_t on a constant and diffC_{t-1}')
X=[ones(totalNpers-2,1) dCall(1:end-1)];
b=X\dCall(2:end)
chiOLS=b(2);

disp('IV Regression of diffC_t on a constant and diffC_{t-1}')
X=[ones(totalNpers-2,1) dCall(1:end-1)];
b=X\dCall(2:end)
dCallFit=X*b;
X=[ones(totalNpers-3,1) dCallFit(1:end-1)];
b=X\dCall(3:end)
chiIV=b(2);

disp('IV Regression of diffC_t on a constant, E_{t-2}diffC_{t-1} and diffB_{t-1}')
X=[ones(totalNpers-2,1) dCall(1:end-1)];
b=X\dCall(2:end)
dCallFit=X*b;
X=[ones(totalNpers-3,1) dCallFit(1:end-1) dBall(2:end-1)];
b=X\dCall(3:end)
chiIVjoint=b(2);
alphaJoint=b(3);

disp('OLS Regression in Levels: C_t on a constant and B_{t-1}')
X=[ones(totalNpers-1,1) BAll_flessRat(1:end-1)];
b=X\CAll_flessRat(2:end)
alphaLevels=b(2);

disp('The truth');
mpc
1-PiFrac
chiTrue=1-PiFrac; alphaTrue=mpc;

% Draw figures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
time=[1:totalNpers]';
fSimAllShoxC=figure; h=plot(time,[CAllRat,CAll_flessRat]); axis tight; axis 'auto y'; 
legend('Sticky C','Frictionless C',2); 
set(h(1),'linewidth',2,'linestyle','-','color','k','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-','color','k');
print(fSimAllShoxC,'-depsc',[basePath 'figures/note/fSimAllShoxC.eps']);
print(fSimAllShoxC,'-dpng',[basePath 'figures/note/fSimAllShoxC.png']);

fSimAllShoxT=figure; h=plot(time,[BAllRat,BAll_flessRat]); axis tight; axis 'auto y'; 
legend('Sticky Total Wealth','Frictionless Total Wealth',3); 
set(h(1),'linewidth',2,'color','k','linestyle','-','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-','color','k');
print(fSimAllShoxT,'-depsc',[basePath 'figures/note/fSimAllShoxT.eps']);
print(fSimAllShoxT,'-dpng',[basePath 'figures/note/fSimAllShoxT.png']);

fSimAllShoxUpdVsNupdC=figure; h=plot([1:nPers]',[CAggrNnRat, CAggrNupdRat, CAggrRat]); axis tight; axis 'auto y';
title('C of Updaters and Nonupdaters Following the Second Shock (to Y growth)');
legend('C of Updaters',['C of Perfect Nonupdaters; Weight: ' num2str(round(fracNonupd*1e3)/1e3)],'Aggregate C',4); 
set(h(1),'linewidth',2,'color','k','linestyle','-','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-','color','k');
set(h(3),'linewidth',2,'linestyle','-','color','r');
%print(fSimAllShoxUpdVsNupdC,'-depsc',[basePath 'figures/note/fSimAllShoxUpdVsNupdC.eps']);
%print(fSimAllShoxUpdVsNupdC,'-dpng',[basePath 'figures/note/fSimAllShoxUpdVsNupdC.png']);

fSimAllShoxUpdVsNupdT=figure; h=plot([1:nPers]',[BAggrNnRat, BAggrNupdRat, BAggrRat]); axis tight; axis 'auto y';
title('Total Wealth of Updaters and Nonupdaters Following the Second Shock (to Y growth)');
legend('Wealth of Updaters',['Wealth of Perfect Nonupdaters; Weight: ' num2str(round(fracNonupd*1e3)/1e3)],'Aggregate Wealth',3); 
set(h(1),'linewidth',2,'color','k','linestyle','-','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-','color','k');
set(h(3),'linewidth',2,'linestyle','-','color','r');
%print(fSimAllShoxUpdVsNupdT,'-depsc',[basePath 'figures/note/fSimAllShoxUpdVsNupdT.eps']);
%print(fSimAllShoxUpdVsNupdT,'-dpng',[basePath 'figures/note/fSimAllShoxUpdVsNupdT.png']);

fSimAllShoxTbig=figure; h=plot(time,[BAllRat,BAll_flessRat,[repmat(nan,nPersPlusStart,1); BAggrNnRat],[repmat(nan,nPersPlusStart,1); BAggrNupdRat]]);
axis tight; axis 'auto y'; 
legend('Sticky Total Wealth','Frictionless Total Wealth','Updaters',['Perfect Nonupdaters; Weight: ' num2str(round(fracNonupd*1e3)/1e3)],3); 
set(h(1),'linewidth',2,'color','k','linestyle','-','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-','color','k');
set(h(3),'linewidth',2,'linestyle','-','color','r');
set(h(4),'linewidth',2,'linestyle',':','color','r');
%print(fSimAllShoxTbig,'-depsc',[basePath 'figures/note/fSimAllShoxTbig.eps']);
%print(fSimAllShoxTbig,'-dpng',[basePath 'figures/note/fSimAllShoxTbig.png']);

%====================================================================
t=toc; mints=floor(toc/60); secs=toc-60*mints;
fprintf('Time: %4.0f minutes, %4.2f seconds\n',mints,secs); diary off;


