
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Carroll, Otsuka and Slacalek: "How Large Are Financial and Housing Wealth Effects?: A New Approach"
% January 30, 2009
% 
% This program simulates the reaction of consumption and wealth to a shock to income growth under frictionless
% and sticky expectations. The experiment is described in simulationsWithStickyC.pdf
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all; 
thispath = mfilename('fullpath'); thisfile = mfilename;
pathlength = size(thispath,2); namelength = size(thisfile,2);
CurrDir = thispath(1:(pathlength-namelength));
basePath=CurrDir(1:( length(CurrDir)-length('theoretical\matlab\') ));
cd(CurrDir);
diaryName=[mfilename '.out']; delete(diaryName);  diary(diaryName); 

disp('==================================================================================');
disp(['Running  ' mfilename '.m                      ' datestr(now)]); format compact; tic;
disp('==================================================================================');
% Baseline parameters 
% Chris's setup
rho=2e3;        % relative risk aversion
g=0;
%R=(1+g)^2/beta;
R=1.05;         % interest factor
beta=1/R;       % discount factor
BLev0=0;        % initial wealth
YLev0=1;        % initial income
nPersStart=5;   % # periods before the sim starts
nPers=11;       % # periods after the sim starts
PiFrac=0.25;    % fraction of updaters;

totalNpers=nPersStart+nPers-1;
r=R-1;
G=1+g;
mpc=(R-(R*beta)^(1/rho))/R;

bigI=nPers; % upper bound on the # of inattention periods over which aggregate C is summed;

% Shock to Income Growth (G) Scenario;
%=========================================================================;
% initialize matrices;
OLev=repmat(nan,totalNpers,totalNpers); % Rows: time periods; columns: inattention cohorts (most attentive on the left);
BLev=OLev; CLev=OLev; YLev=OLev; HLev=OLev;
BLev(1,:)=BLev0; YLev(1,:)=YLev0; 
BactLev=BLev; % Bact stores actual market wealth of the inattentive (not perceived by them until they update)
G1=(1.025)^.25;
G=G0; 

for i=1:totalNpers; % Loop over time;
    for j=1:totalNpers; % Loop over inattention spans
        if i-4==j; %floor(i/5)==(i/5); 
           BLev(i,j)=BactLev(i,j);   % (i);
           YLev(i,j)=YLev(i,j)*((G1/G0)^(j-1)); % effectively (ii);
        end;
        if i-4>=j; G=G1; else; G=G0; end; % (iii) [and (ii)];
        
        HLev(i,j)=YLev(i,j)/(1-G/R);
        OLev(i,j)=BLev(i,j)+HLev(i,j); 
        CLev(i,j)=mpc*OLev(i,j);

        YLev(i+1,j)=G*YLev(i,j);
        BLev(i+1,j)=R*(BLev(i,j)+YLev(i,j)-CLev(i,j));
        BactLev(i+1,j)=R*(BactLev(i,j)+YLev(i,1)-CLev(i,j));    % Note that Y(i,1) is the actual income (of frictionless consumers);
    end;
end;

% Normalize with income;
BRat=BLev./YLev; BRat=CLev./YLev(1:end-1,:); HRat=HLev./YLev(1:end-1,:); ORat=OLev./YLev(1:end-1,:); YRat=YLev./YLev;
t=[1:totalNpers]';

% Sum over consumers to get the effect on aggregate vars;
auxVector=[1; (1-PiFrac)*ones(totalNpers-1,1)];
consWeights=PiFrac*cumprod(auxVector); consWeights=repmat(consWeights',totalNpers,1);
CAggrRat=sum((CRat.*consWeights)')';
dCaggrRat=diff(CAggrRat);
BAggrRat=sum((BRat(1:end-1,:).*consWeights)')';
dBaggrRat=diff(BAggrRat);

% Figures;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NN=min([totalNpers,20]);
fSimStickyGshox=figure; h=plot(t(1:NN),[CAggrRat(1:NN),CRat(1:NN,1) BAggrRat(1:NN) BRat(1:NN,1)]); axis tight; axis 'auto y'; 
legend('Aggregate Consumption under Sticky Expectations','Aggregate Consumption under Frictionless Expectations','Market Wealth under Sticky','Frictionless Market Wealth',3); 
set(h(1),'linewidth',2,'color','k','linestyle','-','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-','color','k');
set(h(3),'linewidth',2,'color','k','linestyle','-.','marker','o'); 
set(h(4),'linewidth',2,'linestyle','-.','color','k');
print(fSimStickyGshox,'-depsc',[basePath 'figures/note/fSimStickyGshox.eps']);

fSimStickyGshoxConly=figure; h=plot(t,[CAggrRat,CRat(:,1)]); axis tight; axis 'auto y'; 
legend('Aggregate Consumption under Sticky Expectations','Aggregate Consumption under Frictionless Expectations',4); 
set(h(1),'linewidth',2,'color','k','linestyle','-','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-','color','k');
print(fSimStickyGshoxConly,'-depsc',[basePath 'figures/note/fSimStickyGshoxConly.eps']);

% Frictionless figure only;
tVec=t;
fSimStickyBshoxFric=figure; h=plot(tVec,[CRat(:,1) BRat(1:end-1,1)]); axis tight; axis 'auto y'; 
xlabel('Quarters');
legend('Consumption under Frictionless Expectations','Frictionless Market Wealth',3); 
set(h(1),'linewidth',2,'linestyle','-','color','k');
set(h(2),'linewidth',2,'linestyle','-.','color','k');
print(fSimStickyBshoxFric,'-depsc',[basePath 'figures/note/PFonlyG.eps']);

%====================================================================
t=toc; mints=floor(toc/60); secs=toc-60*mints;
fprintf('Time: %4.0f minutes, %4.2f seconds\n',mints,secs); diary off;


