
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Carroll, Otsuka and Slacalek: "How Large Are Financial and Housing Wealth Effects?: A New Approach"
% January 30, 2009
% 
% This program simulates (using method 1) the reaction of consumption and wealth to a shock to wealth under frictionless
% and sticky expectations. The experiment is described in simulationsWithStickyC.pdf
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all; 
thispath = mfilename('fullpath'); thisfile = mfilename;
pathlength = size(thispath,2); namelength = size(thisfile,2);
CurrDir = thispath(1:(pathlength-namelength));
basePath=CurrDir(1:( length(CurrDir)-length('theoretical\matlab\') )); cd(CurrDir);
diaryName=[mfilename '.out']; delete(diaryName);  diary(diaryName); 

disp('==================================================================================');
disp(['Running  ' mfilename '.m                      ' datestr(now)]); format compact; tic;
disp('==================================================================================');

% Baseline parameters 
% Chris's setup
rho=2e3;        % relative risk aversion
g=0;
%R=(1+g)^2/beta;
R=1.05;         % interest factor
beta=1/R;       % discount factor
BLev0=0;        % initial wealth
YLev0=1;        % initial income
nPersStart=5;   % # periods before the sim starts
nPers=11;       % # periods after the sim starts
PiFrac=0.25;    % fraction of updaters;

totalNpers=nPersStart+nPers-1;
r=R-1;
G=1+g;
mpc=(R-(R*beta)^(1/rho))/R;

bigI=nPers; % upper bound on the # of inattention periods over which aggregate C is summed;

%====================================================================
% Shox to market wealth (B)
%====================================================================
% initialize matrices;
BLevShock=1; % times initial income
OLev=repmat(nan,totalNpers,1);
BLev=OLev; CLev=OLev; YLev=OLev; HLev=OLev;
BLev(1)=BLev0; YLev(1)=YLev0;

BbarLev=OLev; CbarLev=OLev; CratLev=OLev; YbarLev=OLev; HbarLev=OLev; ObarLev=OLev; 
BbarLev(1)=BLev0; YbarLev(1)=YLev0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switchIndicator=1;

for i=1:totalNpers; % Loop over time;

    if floor((i-nPersStart)/(nPers))==((i-nPersStart)/(nPers)) %i==nPersStart; %
        BLev(i)=BLev(i)+switchIndicator*BLevShock;            
        BbarLev(i)=BbarLev(i)+switchIndicator*BLevShock;
        switchIndicator=-switchIndicator;
    end;

    HLev(i)=YLev(i)/(1-G/R);
    OLev(i)=BLev(i)+HLev(i);
    CLev(i)=mpc*OLev(i);

    YLev(i+1)=G*YLev(i);
    BLev(i+1)=R*(BLev(i)+YLev(i)-CLev(i));
    
    HbarLev(i)=YbarLev(i)/(1-G/R);
    ObarLev(i)=BbarLev(i)+HbarLev(i);
    CratLev(i)=mpc*ObarLev(i);    % rational C for those living in the sticky world
    
    if i==1; CbarLev(i)=CLev(i); 
    else; CbarLev(i)=PiFrac*CratLev(i)+(1-PiFrac)*((R*beta)^(1/rho))*CbarLev(i-1);
    end;
    
    YbarLev(i+1)=G*YbarLev(i);
    BbarLev(i+1)=R*(BbarLev(i)+YbarLev(i)-CbarLev(i));
end;

% Normalize with income;
CratRat=CratLev./YLev(1:end-1); CRat=CLev./YLev(1:end-1); BRat=BLev(1:end-1)./YLev(1:end-1);
CbarRat=CbarLev./YLev(1:end-1); BbarRat=BbarLev(1:end-1)./YLev(1:end-1); ObarRat=ObarLev./YLev(1:end-1);

% Figures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
time=[1:totalNpers]';
fSimWealthShoxC=figure; h=plot(time,[CbarRat,CratRat,CRat]); axis tight; axis 'auto y'; 
legend('Sticky C','Rational C in the Sticky World','Frictionless C',4); 
set(h(1),'linewidth',2,'linestyle','-','color','k','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-.','color','k');
set(h(3),'linewidth',2,'linestyle',':','color','k');
print(fSimWealthShoxC,'-depsc',[basePath 'figures/note/fSimWealthShoxC.eps']);
print(fSimWealthShoxC,'-dpng',[basePath 'figures/note/fSimWealthShoxC.png']);

fSimWealthShoxB=figure; h=plot(time,[BbarRat,BRat]); axis tight; axis 'auto y'; 
legend('Sticky Total Wealth','Frictionless Total Wealth',4); 
set(h(1),'linewidth',2,'color','k','linestyle','-','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-.','color','k');
print(fSimWealthShoxB,'-depsc',[basePath 'figures/note/fSimWealthShoxT.eps']);
print(fSimWealthShoxB,'-dpng',[basePath 'figures/note/fSimWealthShoxT.png']);

%====================================================================
time=toc; mints=floor(toc/60); secs=toc-60*mints;
fprintf('Time: %4.0f minutes, %4.2f seconds\n',mints,secs); diary off;
