
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Carroll, Otsuka and Slacalek: "How Large Are Financial and Housing Wealth Effects?: A New Approach"
% January 30, 2009
% 
% This program simulates (using method 2) the reaction of consumption and wealth to a shock to wealth under frictionless
% and sticky expectations. The experiment is described in simulationsWithStickyC.pdf
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all; 
thispath = mfilename('fullpath'); thisfile = mfilename;
pathlength = size(thispath,2); namelength = size(thisfile,2);
CurrDir = thispath(1:(pathlength-namelength));
basePath=CurrDir(1:( length(CurrDir)-length('theoretical\matlab\') )); cd(CurrDir);
diaryName=[mfilename '.out']; delete(diaryName);  diary(diaryName); 

disp('==================================================================================');
disp(['Running  ' mfilename '.m                      ' datestr(now)]); format compact; tic;
disp('==================================================================================');
% Baseline parameters 
% Chris's setup
rho=2e3;        % relative risk aversion
g=0;
%R=(1+g)^2/beta;
R=1.05;         % interest factor
beta=1/R;       % discount factor
BLev0=0;        % initial wealth
YLev0=1;        % initial income
nPersStart=5;   % # periods before the sim starts
nPers=11;       % # periods after the sim starts
PiFrac=0.25;    % fraction of updaters;

totalNpers=nPersStart+nPers-1;
r=R-1;
G=1+g;
mpc=(R-(R*beta)^(1/rho))/R;

bigI=nPers; % upper bound on the # of inattention periods over which aggregate C is summed;

% Shock to B scenario;
%=========================================================================;
% initialize matrices;
BLevShock=1; % times initial income ("Lev"-variables denote simple levels [not ratios to permanent income])
OLev=repmat(nan,totalNpers,totalNpers); % Rows: time periods; columns: inattention cohorts (most attentive on the left);
BLev=OLev; CLev=OLev; YLev=OLev; HLev=OLev; BactLev=OLev;
BLev(1,:)=BLev0; BactLev(1,:)=BLev0; YLev(1,:)=YLev0; 
switchIndicator=1;
Rvec=R*ones(totalNpers,1); Rcum=[1; cumprod(Rvec)]; % Rcum: accumulated interest rate factor

for i=1:totalNpers; % Loop over time;
    for j=1:totalNpers; % Loop over inattention spans
        if i-4==j; %floor(i/5)==(i/5); 
            BLev(i,j)=BLev(i,j)+(BLevShock)*R^(j-1);
            %switchIndicator=-switchIndicator;
        end;
        
        if i==5;
            BactLev(i,j)=BactLev(i,j)+BLevShock;
        end;

        HLev(i,j)=YLev(i,j)/(1-G/R);
        OLev(i,j)=BLev(i,j)+HLev(i,j); 
        CLev(i,j)=mpc*OLev(i,j);

        YLev(i+1,j)=G*YLev(i,j);
        BLev(i+1,j)=R*(BLev(i,j)+YLev(i,j)-CLev(i,j));
        BactLev(i+1,j)=R*(BactLev(i,j)+YLev(i,j)-CLev(i,j));
    end;
end;

% Normalize with income;
bRat=BLev./YLev; cRat=CLev./YLev(1:end-1,:); hRat=HLev./YLev(1:end-1,:); oRat=OLev./YLev(1:end-1,:); yRat=YLev./YLev; bRatAct=BactLev./YLev; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Sum over consumers to get the effect on aggregate vars;
auxVector=[1; (1-PiFrac)*ones(totalNpers-1,1)];
consWeights=PiFrac*cumprod(auxVector); consWeights=repmat(consWeights',totalNpers,1);
cRatAggr=sum((cRat.*consWeights)')'+(1-sum(consWeights(1,:)))*cRat(:,end);
bRatAggr=sum((bRat(1:end-1,:).*consWeights)')'; %+(1-sum(consWeights(1,:)))*b(1:end-1,end); % not necessary to adjust as b(:,end) is zero
bRatActAggr=sum((bRatAct(1:end-1,:).*consWeights)')'+(1-sum(consWeights(1,:)))*bRatAct(1:end-1,end);

% Figures;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t=[1:totalNpers]';
fSimStickyBshox=figure; h=plot(t,[cRatAggr,cRat(:,1) bRatAggr bRat(1:end-1,1) bRatActAggr]); axis tight; axis 'auto y'; 
legend('Aggregate Consumption under Sticky Expectations','Aggregate Consumption under Frictionless Expectations','Market Wealth under Sticky',...
    'Frictionless Market Wealth','Actual Market Wealth',4); 
xlabel('Quarters'); 
set(h(1),'linewidth',2,'color','k','linestyle','-','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-','color','k');
set(h(3),'linewidth',2,'color','k','linestyle','-.','marker','o'); 
set(h(4),'linewidth',2,'linestyle','-.','color','k');
print(fSimStickyBshox,'-depsc',[basePath 'figures/note/fSimStickyBshox.eps']);
print(fSimStickyBshox,'-dpng',[basePath 'figures/note/fSimStickyBshox.png']);

fSimStickyBshoxConly=figure; h=plot(t,[cRatAggr,cRat(:,1)]); axis tight; axis 'auto y'; 
legend('Aggregate Consumption under Sticky Expectations','Aggregate Consumption under Frictionless Expectations',4); 
set(h(1),'linewidth',2,'color','k','linestyle','-','marker','o'); 
set(h(2),'linewidth',2,'linestyle','-','color','k');
print(fSimStickyBshoxConly,'-depsc',[basePath 'figures/note/fSimStickyBshoxConly.eps']);
print(fSimStickyBshoxConly,'-dpng',[basePath 'figures/note/fSimStickyBshoxConly.png']);

% Frictionless figure only;
fSimStickyBshoxFric=figure; h=plot(t,[cRat(:,1) bRat(1:end-1,1)]); axis tight; axis 'auto y'; 
xlabel('Quarters');
legend('Consumption under Frictionless Expectations','Frictionless Market Wealth',4); 
set(h(1),'linewidth',2,'linestyle','-','color','k');
set(h(2),'linewidth',2,'linestyle','-.','color','k');
print(fSimStickyBshoxFric,'-depsc',[basePath 'figures/note/PFonlyW.eps']);
print(fSimStickyBshoxFric,'-dpng',[basePath 'figures/note/PFonlyW.png']);

%====================================================================
t=toc; mints=floor(toc/60); secs=toc-60*mints;
fprintf('Time: %4.0f minutes, %4.2f seconds\n',mints,secs); diary off;

